Socket.io may or may not work with Nginx out-of-the-box. Most approaches tries to make Nginx open a proxy connection to Socket.io, which fails on Nginx 1.0 because it does not support proxying of HTTP/1.1 connections.

Socket.io and Nginx 1.0 (recommended)
-------------------------------------

Jonathan Leppert has an article on how to use the Nginx `tcp_proxy` module to work with Socket.io.

* http://www.letseehere.com/reverse-proxy-web-sockets

Socket.io and Nginx 1.1
-----------------------

The unstable branch, Nginx 1.1, has support for proxying HTTP/1.1 connections. Simply use `proxy_pass`.

__NOTE:__ At time of writing (v1.1.4), Nginx 1.1 still can't support web sockets.

``` nginx
server {
  listen 80;
  server_name socket.myserver.com;
  location / {
    proxy_pass http://localhost:3030;   # <-- this is where your Socket.io server is listening at
  }
}
```

To install Nginx 1.1 in most systems, you will probably need to compile it from source. Information on how to do this is available from the Nginx website.

* http://wiki.nginx.org/Install#Source_Releases

Socket.io and Phusion Passenger 3 + Nginx
-----------------------------------------

Using Passenger's `passenger-install-nginx-module` installs a stable branch of Nginx. If you would like to use the development branch (v1.1 at time of writing), you will need to download the sources yourself.

``` shell
$ mkdir -p /tmp/src/nginx
$ cd /tmp/src/nginx
$ wget -O - http://nginx.org/download/nginx-1.1.1.tar.gz | tar xz --strip-components=1
```

Then, invoke `passenger-install-nginx-module` with the `--nginx-source-dir` option.

``` shell
$ passenger-install-nginx-module --nginx-source-dir=/tmp/src/nginx
```