After finding out that nearly 3% out of 100.000 requests were from beyond a firewall and thus unable to reach our Node instance on port 4000, we needed to come up with an alternative (fallback) port. However, there's personal firewalls, and some of them are known to block WebSockets.

Time, therefore, to test which firewall eats what.

### Test environment

All tests were done in a Virtual Machine (parallels 6) running Windows XP. Only one of the software packages where installed per test and the Virtual Machine was rolled back to a clean snapshot after each run. The software was tested with the default settings and the "zomg paranoid panix button" settings.

I've done two tests:

* with no special firewall rules on the host machine
* with port 4000 blocked on the host machine, to emulate the VM being behind some corporate firewall.

The client js will initially try to connect to port 4000 and fall back to ports 80 or 843, if that doesn't work. I also tested different orders for port 80 and 843, to make sure both work (or not).

Browser cache was disabled, cookies cleared between each testing run. I used Chrome and Firefox (3.6). Where possible, I checked in IE as well, but since IE doesn't kill cookies when you tell it to, I mostly tried in browsers that do. This exercise is painful enough as it is...

## TL;DR

* Non-WebSockets connections were never an issue
* WebSockets are not blocked by most firewalls. (Only Avast, AVG and BitDefender block)
* Most blocking firewalls *do* allow WebSockets on port 843

## Test Results

In alphabetical order:

* [AVG Internet Security 2012](#sdflshfdkhsdflhjsdflhjsdf-avg)
* [Avast Internet Security](#sdflshfdkhsdflhjsdflhjsdf-avast)
* [Avira Antivirus Premium](#sdflshfdkhsdflhjsdflhjsdf-avira)
* [Bitdefender Total Security 2012](#sdflshfdkhsdflhjsdflhjsdf-bitdefender)
* [Comodo Internet Security Pro](#sdflshfdkhsdflhjsdflhjsdf-comodo)
* [ESET Smart Security 5](#sdflshfdkhsdflhjsdflhjsdf-eset)
* [Kaspersky Internet Security](#sdflshfdkhsdflhjsdflhjsdf-kaspersky)
* [McAfee Total Protection 2011](#sdflshfdkhsdflhjsdflhjsdf-mcafee)
* [Microsoft Security Essentials](#sdflshfdkhsdflhjsdflhjsdf-mse)
* [Norton Internet Security 2011](#sdflshfdkhsdflhjsdflhjsdf-norton)
* [Online Armor++](#sdflshfdkhsdflhjsdflhjsdf-onlinearmor)
* [Panda Internet Security](#sdflshfdkhsdflhjsdflhjsdf-panda)
* [Privoxy](#sdflshfdkhsdflhjsdflhjsdf-privoxy)
* [Trend Micro Titanium Maximum Security 2012](#sdflshfdkhsdflhjsdflhjsdf-trendmicro)
* [Windows Defender](#sdflshfdkhsdflhjsdflhjsdf-windefender)
* [ZoneAlarm Extreme Security](#sdflshfdkhsdflhjsdflhjsdf-za)


---

<h3 id="sdflshfdkhsdflhjsdflhjsdf-windefender">Windows Defender</h3>

* Chrome can do its WebSockets magic through any available port
* Other browsers can connect through any available port

<h3 id="sdflshfdkhsdflhjsdflhjsdf-mse">Microsoft Security Essentials</h3>

* Chrome can do its WebSockets magic through any available port
* Other browsers can connect through any available port

<h3 id="sdflshfdkhsdflhjsdflhjsdf-avast">Avast Internet Security</h3>

The only change I made was removing the local network from the friends list.
Production sites tend to be outside that range as well.

* With no firewall active, all browsers connect to port 4000
* FF / IE, firewall blocks 4000: both 80 and 843 work
* Chrome, FW: Oh dear, Avast eats our WebSocket at port 80. This is what got me to add port 843 to the list of options in the first place (well, @3rd-Eden told me, actually). Lo and behold: port 843 *works*.

Note, this research was done using avast 6.0.1289 or older. The version 6.0.1367 seems to fix at least some of the issues.

<h3 id="sdflshfdkhsdflhjsdflhjsdf-mcafee">McAfee Total Protection 2011</h3>

Interestingly enough, McAfee notified me of a Trojan my VM appeared to have, and that Windows Defender and Avast never mentioned. Interesting.

* With no firewall active, all browsers connect to port 4000.  
* Firewall on: Both 80 and 843 work, for all browsers.

<h3 id="sdflshfdkhsdflhjsdflhjsdf-norton">Norton Internet Security 2011</h3>

* Chrome can do its WebSockets magic through any available port
* Other browsers can connect through any available port

<h3 id="sdflshfdkhsdflhjsdflhjsdf-eset">ESET Smart Security 5</h3>

* Chrome can do its WebSockets magic through any available port
* Other browsers can connect through any available port
* It does however block WebSocket connections from localhost

<h3 id="sdflshfdkhsdflhjsdflhjsdf-za">ZoneAlarm Extreme Security</h3>

* Chrome can do its WebSockets magic through any available port
* Other browsers can connect through any available port

<h3 id="sdflshfdkhsdflhjsdflhjsdf-panda">Panda Internet Security</h3>

* Chrome can do its WebSockets magic through any available port
* Other browsers can connect through any available port

<h3 id="sdflshfdkhsdflhjsdflhjsdf-comodo">Comodo Internet Security Pro
2011</h3>

Interestingly enough, Comodo cried wolf about the entries in the hosts file that Parallels put there, classing it as a trojan. It also claimed to find a worm in Silverlight that no other package had found.

I assume Comodo's free firewall is the same as the one included in this package, so I didn't test it separately.

* Chrome can do its WebSockets magic through any available port
* Other browsers can connect through any available port

<h3 id="sdflshfdkhsdflhjsdflhjsdf-kaspersky">Kaspersky Internet Security 2011</h3>

* Chrome can do its WebSockets magic through any available port
* Other browsers can connect through any available port

<h3 id="sdflshfdkhsdflhjsdflhjsdf-avg">AVG Internet Security 2012</h3>

The first one to ask questions about the browser opening connections to Node on port 4000. After that it's smooth sailing.

* No questions asked about port 80 or 843 under Firefox
* WebSockets get blocked on *all* ports, despite AVG's logs showing the connection was allowed.
* Non-websocket transports work fine, apart from the initial question whether the user wants to allow their browser to connect to the internet.

<h3 id="sdflshfdkhsdflhjsdflhjsdf-avira">Avira Antivirus Premium</h3>

The trial version wouldn't let me enable the firewall... There is some web protection stuff in there, but it doesn't block any connectivity.

<h3 id="sdflshfdkhsdflhjsdflhjsdf-bitdefender">Bitdefender Total Security 2012</h3>

Bitdefender has a fair bit of options to make the firewall super paranoid.
Wonderful, let's switch on ALL the options... =]

* Allows Firefox to connect on any port
* WebSockets are blocked, but Chrome falls back to XHR-polling (interestingly enough, this does not happen with other antivirus products; there, Chrome just fails to connect. Might be an issue with my test code). Bitdefender logs it as stealth traffic.

<h3 id="sdflshfdkhsdflhjsdflhjsdf-trendmicro">Trend Micro Titanium Maximum Security 2012</h3>

* Chrome can do its WebSockets magic through any available port
* Other browsers can connect through any available port

<h3 id="sdflshfdkhsdflhjsdflhjsdf-onlinearmor">Online Armor++</h3>

* Chrome can do its WebSockets magic through any available port
* Other browsers can connect through any available port

<h3 id="sdflshfdkhsdflhjsdflhjsdf-privoxy">Privoxy</h3>

I used Privoxy's default settings. Since the user can cook up any rule they want, it *would* be possible to block the handshake, I think, but I don't consider it very likely.

* Chrome can do its WebSockets magic through any available port
* Other browsers can connect through any available port